package Unix::Supervise;

use strict;
use Carp;

#
# Copyright (c) 2003 James Raftery <james@now.ie>. All rights reserved.
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
# Please submit bug reports, patches and comments to the author.
# Latest information at http://romana.now.ie/#net-qmtp
#
# $Id: Supervise.pm,v 1.3 2003/02/03 18:38:10 james Exp $
#
# This module is an object interface to svscan/supervise monitored
# unix services.
#
# See the Unix::Supervise man page that was installed with this module for
# information on how to use the module.
#

use vars qw($VERSION);
$VERSION = '0.01';

delete @ENV{'IFS', 'CDPATH', 'ENV', 'BASH_ENV'};
$ENV{'PATH'} = '/bin:/usr/bin:/usr/local/bin:/command:/usr/local/sbin';

sub new {
	my $proto = shift or croak;
	my $class = ref($proto) || $proto;
	my $dir = shift or croak "No directory specified in constructor";
	my %args;

	%args = @_ if @_;

	# new object
	# parameters: service directory, path to svc, svok
	# bless reference and return

	my $self = {
		DIR		=> $dir,
		DEBUG		=> undef,
		DIRHANDLE	=> undef,
	};

	if ($args{'Debug'}) {
		$self->{DEBUG} = 1;
		warn "debugging on; Version: $VERSION; RCS " .
			qq$Revision: 1.3 $ . "\n";
	}
	if ($args{'Path'}) {
		# leading/trailing colons have nasty consequences
		$args{'Path'} =~ s/^:+//;
		$args{'Path'} =~ s/:+$//;
		$ENV{'PATH'} .= ":" . $args{'Path'};
	}

	if (!-d $dir) {
		carp "$dir is not a directory";
		return undef;
	}

	if (!chdir $dir) {
		carp "Cannot chdir to $dir: $!";
		return undef;
	}

	if (!opendir(DIR, ".")) {
		carp "Cannot open $dir: $!";
		return undef;
	}
	$self->{DIRHANDLE} = *DIR;

	bless($self, $class);
	return $self;
}

sub DESTROY {
	my $self = shift;
	ref($self) or croak;

	local *DIR = $self->{DIRHANDLE};
	my $dir = $self->{DIR};

	if (fileno DIR) {
		closedir DIR or carp "Cannot close $dir: $!";
	}
}

sub services {
	my $self = shift;
	ref($self) or croak;

	my %r;
	foreach (@{ $self->_list_services() }) {
		$r{$_} = $self->status($_);
	}
	return \%r;
}

sub _list_services {
	my $self = shift;
	ref($self) or croak;

	local *DIR = $self->{DIRHANDLE};
	my $dir = $self->{DIR};
	my @r;

	# open service dir., read, check for sticky and DIR/log
	# no dot-files and no funny characters
	foreach (grep { /^[a-z0-9_:,-]+$/ } (grep { /^[^.]/ } readdir DIR)) {
		if ($self->{DEBUG}) {
			print "$dir: $_ (";
			print " is sticky" if -k $_;
			print " is symlink" if -l $_;
			print " contains log" if -d "$_/log";
			print " )\n";
		}
		push(@r, $_);
		push(@r, "$_/log") if -d "$_/log";
	}
	return \@r;
}

sub up {
	my $self = shift;
	ref($self) or croak;

	my $service = shift or croak "No service name supplied";
	system("svc -u $service") == 0 or carp "up failed: exit " . $? >> 8;
}

sub down {
	my $self = shift;
	ref($self) or croak;

	my $service = shift or croak "No service name supplied";
	system("svc -d $service") == 0 or carp "down failed: exit " . $? >> 8;
}

sub once {
	my $self = shift;
	ref($self) or croak;

	my $service = shift or croak "No service name supplied";
	system("svc -o $service") == 0 or carp "once failed: exit " . $? >> 8;
}

sub exit {
	my $self = shift;
	ref($self) or croak;

	my $service = shift or croak "No service name supplied";
	system("svc -x $service") == 0 or carp "exit failed: exit " . $? >> 8;
}

sub sighup {
	my $self = shift;
	ref($self) or croak;

	my $service = shift or croak "No service name supplied";
	system("svc -h $service") == 0 or carp "hup failed: exit " . $? >> 8;
}

sub sigterm {
	my $self = shift;
	ref($self) or croak;

	my $service = shift or croak "No service name supplied";
	system("svc -t $service") == 0 or carp "term failed: exit " . $? >> 8;
}

sub sigkill {
	my $self = shift;
	ref($self) or croak;

	my $service = shift or croak "No service name supplied";
	system("svc -k $service") == 0 or carp "kill failed: exit " . $? >> 8;
}

sub sigstop {
	my $self = shift;
	ref($self) or croak;

	my $service = shift or croak "No service name supplied";
	system("svc -p $service") == 0 or carp "stop failed: exit " . $? >> 8;
}

sub sigcont {
	my $self = shift;
	ref($self) or croak;

	my $service = shift or croak "No service name supplied";
	system("svc -c $service") == 0 or carp "cont failed: exit " . $? >> 8;
}

sub sigalrm {
	my $self = shift;
	ref($self) or croak;

	my $service = shift or croak "No service name supplied";
	system("svc -a $service") == 0 or carp "alrm failed: exit " . $? >> 8;
}

sub sigint {
	my $self = shift;
	ref($self) or croak;

	my $service = shift or croak "No service name supplied";
	system("svc -i $service") == 0 or carp "int failed: exit " . $? >> 8;
}

sub status {
	my $self = shift;
	ref($self) or croak;

	my $service = shift or croak "No service name supplied";
	chomp($service = `svstat $service`);
	return $service;
}

sub ok {
	my $self = shift;
	ref($self) or croak;

	my $service = shift or croak;
	if (system("svok $service") == 0) {
		return 1;
	} else {
		return 0;
	}
}

1;

__END__

=head1 NAME

Unix::Supervise - Perl interface to svscan/supervise monitored services

=head1 SYNOPSIS

 #!/usr/bin/perl -Tw
 use Unix::Supervise;

 $svc = Unix::Supervise->new("/service");

 $svc->sigterm("dnscache");
 $svc->sigalrm("qmail-send");
 $svc->down("qmail-smtpd");

=head1 DESCRIPTION

This module is a object interface to services managed by the
svscan(8) and supervise(8) tools, from Dan J. Bernstein's daemontools
package. The methods are wrappers around the svc(8), svstat(8) and
svok(8) programs.

=head2 CONSTRUCTOR

=over 4

=item new(DIR [, OPTIONS])

Creates a new Unix::Supervise object to control the services in the
supplied service directory, C<DIR>. C<DIR> is mandatory. new() also
accepts options supplied as a list of hash key/value pairs. Available
options are:

B<Path> - a colon seperated string containing extra directories to be
added to the default PATH,
C</bin:/usr/bin:/usr/local/bin:/command:/usr/local/sbin>

B<Debug> - set to 1 to enable debugging output

=back

=head2 CONTROL METHODS

=over 4

=item up(SERVICE)

Bring C<SERVICE> up. If the service is not running, start it. If the
service stops, restart it.

=item down(SERVICE)

Bring C<SERVICE> down. If the service is running, send it a TERM signal
and then a CONT signal. After it stops, do not restart it.

=item once(SERVICE)

Bring C<SERVICE> up once. If the service is not running, start it. Do
not restart it if it stops.

=item exit(SERVICE)

supervise(8) will exit as soon as the service is down. You will need to
explicitly call down() to bring down the service. svscan(8) may restart
supervise(8).

=back

=head2 STATUS METHODS

=over 4

=item status(SERVICE)

Returns the human-readable status of C<SERVICE>.

=item services()

Returns a reference to a hash. They hash keys are service names and
the hash value is the status for each service.

=item ok(SERVICE)

Returns C<TRUE> if supervise(8) is successfully running for C<SERVICE>,
C<FLASE> otherwise.

=back

=head2 SIGNAL METHODS

=over 4

=item sigstop(SERVICE)

Pause -- send the service a STOP signal.

=item sigcont(SERVICE)

Continue -- send the service a CONT signal.

=item sighup(SERVICE)

Hangup -- send the service a HUP signal.

=item sigalrm(SERVICE)

Alarm -- send the service an ALRM signal.

=item sigint(SERVICE)

Interrupt -- send the service an INT signal.

=item sigterm(SERVICE)

Terminate -- send the service a TERM signal.

=item sigkill(SERVICE)

Kill -- send the service a KILL signal.

=back

=head1 SEE ALSO

L<svc(8)>, L<svok(8)>, L<svstat(8)>, L<supervise(8)>, L<svscan(8)>.

=head1 NOTES

More information about daemontools is available from
http://cr.yp.to/daemontools.html

=head1 CAVEATS

Run any perl program that makes use of this module under the C<-T> flag.
The module makes extensive use of external programs. Please use C<-T> --
if you don't you may just shoot yourself in the foot.

=head1 BUGS

This is alpha code; probably lots.

=head1 AUTHOR

James Raftery <james@now.ie>.

=cut
